@@ -91,7 +91,14 @@ def file_hash(filename):
     path = os.path.normpath(filename.encode("UTF-8"))
     m = hashlib.md5(str(len(path)) + ":")
     m.update(path)
-    m.update(open(filename).read())
+    f = open(filename, 'rb')
+    try:
+        data = f.read(65000)
+        while data:
+            m.update(data)
+            data = f.read(65000)
+    finally:
+        f.close()
     return m.hexdigest()
 
 def parse_list(s):
@@ -671,7 +678,7 @@ def cythonize(module_list, exclude=[], nthreads=0, aliases=None, quiet=False, fo
         options['include_path'] = ['.']
     if 'common_utility_include_dir' in options:
         if options.get('cache'):
-            raise NotImplementedError, "common_utility_include_dir does not yet work with caching"
+            raise NotImplementedError("common_utility_include_dir does not yet work with caching")
         if not os.path.exists(options['common_utility_include_dir']):
             os.makedirs(options['common_utility_include_dir'])
     c_options = CompilationOptions(**options)
